@@ -25,8 +25,10 @@
 #undef dout_prefix
 #define dout_prefix *_dout << "librados: "
 
-librados::IoCtxImpl::IoCtxImpl()
-  : aio_write_list_lock("librados::IoCtxImpl::aio_write_list_lock")
+librados::IoCtxImpl::IoCtxImpl() :
+  ref_cnt(0), client(NULL), poolid(0), assert_ver(0), notify_timeout(30),
+  aio_write_list_lock("librados::IoCtxImpl::aio_write_list_lock"),
+  aio_write_seq(0), lock(NULL), objecter(NULL)
 {
 }
 
@@ -102,15 +104,19 @@ int librados::IoCtxImpl::snap_create(const char *snapName)
   Mutex mylock ("IoCtxImpl::snap_create::mylock");
   Cond cond;
   bool done;
+  Context *onfinish = new C_SafeCond(&mylock, &cond, &done, &reply);
   lock->Lock();
-  objecter->create_pool_snap(poolid,
-			     sName,
-			     new C_SafeCond(&mylock, &cond, &done, &reply));
+  reply = objecter->create_pool_snap(poolid, sName, onfinish);
   lock->Unlock();
 
-  mylock.Lock();
-  while(!done) cond.Wait(mylock);
-  mylock.Unlock();
+  if (reply < 0) {
+    delete onfinish;
+  } else {
+    mylock.Lock();
+    while (!done)
+      cond.Wait(mylock);
+    mylock.Unlock();
+  }
   return reply;
 }
 
@@ -121,17 +127,22 @@ int librados::IoCtxImpl::selfmanaged_snap_create(uint64_t *psnapid)
   Mutex mylock("IoCtxImpl::selfmanaged_snap_create::mylock");
   Cond cond;
   bool done;
-  lock->Lock();
+  Context *onfinish = new C_SafeCond(&mylock, &cond, &done, &reply);
   snapid_t snapid;
-  objecter->allocate_selfmanaged_snap(poolid, &snapid,
-				      new C_SafeCond(&mylock, &cond, &done, &reply));
+  lock->Lock();
+  reply = objecter->allocate_selfmanaged_snap(poolid, &snapid, onfinish);
   lock->Unlock();
 
-  mylock.Lock();
-  while (!done) cond.Wait(mylock);
-  mylock.Unlock();
-  if (reply == 0)
-    *psnapid = snapid;
+  if (reply < 0) {
+    delete onfinish;
+  } else {
+    mylock.Lock();
+    while (!done)
+      cond.Wait(mylock);
+    mylock.Unlock();
+    if (reply == 0)
+      *psnapid = snapid;
+  }
   return reply;
 }
 
@@ -143,15 +154,19 @@ int librados::IoCtxImpl::snap_remove(const char *snapName)
   Mutex mylock ("IoCtxImpl::snap_remove::mylock");
   Cond cond;
   bool done;
+  Context *onfinish = new C_SafeCond(&mylock, &cond, &done, &reply);
   lock->Lock();
-  objecter->delete_pool_snap(poolid,
-			     sName,
-			     new C_SafeCond(&mylock, &cond, &done, &reply));
+  reply = objecter->delete_pool_snap(poolid, sName, onfinish);
   lock->Unlock();
 
-  mylock.Lock();
-  while(!done) cond.Wait(mylock);
-  mylock.Unlock();
+  if (reply < 0) {
+    delete onfinish; 
+  } else {
+    mylock.Lock();
+    while(!done)
+      cond.Wait(mylock);
+    mylock.Unlock();
+  }
   return reply;
 }
 
@@ -729,9 +744,7 @@ int librados::IoCtxImpl::aio_sparse_read(const object_t oid,
   if (len > (size_t) INT_MAX)
     return -EDOM;
 
-  C_aio_sparse_read_Ack *onack = new C_aio_sparse_read_Ack(c);
-  onack->m = m;
-  onack->data_bl = data_bl;
+  C_aio_sparse_read_Ack *onack = new C_aio_sparse_read_Ack(c, data_bl, m);
   eversion_t ver;
 
   c->io = this;
@@ -816,6 +829,28 @@ int librados::IoCtxImpl::aio_write_full(const object_t &oid,
   return 0;
 }
 
+int librados::IoCtxImpl::aio_remove(const object_t &oid, AioCompletionImpl *c)
+{
+  utime_t ut = ceph_clock_now(client->cct);
+
+  /* can't write to a snapshot */
+  if (snap_seq != CEPH_NOSNAP)
+    return -EROFS;
+
+  c->io = this;
+  queue_aio_write(c);
+
+  Context *onack = new C_aio_Ack(c);
+  Context *onsafe = new C_aio_Safe(c);
+
+  Mutex::Locker l(*lock);
+  objecter->remove(oid, oloc,
+		   snapc, ut, 0,
+		   onack, onsafe, &c->objver);
+
+  return 0;
+}
+
 int librados::IoCtxImpl::remove(const object_t& oid)
 {
   utime_t ut = ceph_clock_now(client->cct);
@@ -1529,8 +1564,10 @@ void librados::IoCtxImpl::C_aio_Ack::finish(int r)
 
 /////////////////////// C_aio_sparse_read_Ack //////////////////////////
 
-librados::IoCtxImpl::C_aio_sparse_read_Ack::C_aio_sparse_read_Ack(AioCompletionImpl *_c)
-  : c(_c)
+librados::IoCtxImpl::C_aio_sparse_read_Ack::C_aio_sparse_read_Ack(AioCompletionImpl *_c,
+								  bufferlist *data,
+								  std::map<uint64_t, uint64_t> *extents)
+  : c(_c), data_bl(data), m(extents)
 {
   c->get();
 }
