@@ -1,8 +1,10 @@
-// -*- mode:C++; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*- 
+// -*- mode:C++; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*-
 // vim: ts=8 sw=2 smarttab
 
+#include "cls/lock/cls_lock_client.h"
 #include "include/buffer.h"
 #include "include/encoding.h"
+#include "include/rbd_types.h"
 
 #include "cls_rbd_client.h"
 
@@ -22,6 +24,7 @@ namespace librbd {
       ::encode(snap, bl);
       op.exec("rbd", "get_size", bl);
       op.exec("rbd", "get_object_prefix", empty);
+      
 
       bufferlist outbl;
       int r = ioctx->operate(oid, &op, &outbl);
@@ -31,8 +34,10 @@ namespace librbd {
       try {
 	bufferlist::iterator iter = outbl.begin();
 	uint64_t size;
+	// get_size
 	::decode(*order, iter);
 	::decode(size, iter);
+	// get_object_prefix
 	::decode(*object_prefix, iter);
       } catch (const buffer::error &err) {
 	return -EBADMSG;
@@ -44,24 +49,32 @@ namespace librbd {
     int get_mutable_metadata(librados::IoCtx *ioctx, const std::string &oid,
 			     uint64_t *size, uint64_t *features,
 			     uint64_t *incompatible_features,
-                             std::set<std::pair<std::string, std::string> > *lockers,
+			     map<rados::cls::lock::locker_id_t,
+				 rados::cls::lock::locker_info_t> *lockers,
                              bool *exclusive_lock,
-			     ::SnapContext *snapc)
+			     string *lock_tag,
+			     ::SnapContext *snapc,
+			     parent_info *parent)
     {
       assert(size);
       assert(features);
       assert(incompatible_features);
+      assert(lockers);
+      assert(exclusive_lock);
       assert(snapc);
+      assert(parent);
 
       librados::ObjectReadOperation op;
-      bufferlist sizebl, featuresbl, empty;
+      bufferlist sizebl, featuresbl, parentbl, empty;
       snapid_t snap = CEPH_NOSNAP;
       ::encode(snap, sizebl);
       ::encode(snap, featuresbl);
+      ::encode(snap, parentbl);
       op.exec("rbd", "get_size", sizebl);
       op.exec("rbd", "get_features", featuresbl);
       op.exec("rbd", "get_snapcontext", empty);
-      op.exec("rbd", "list_locks", empty);
+      op.exec("rbd", "get_parent", parentbl);
+      rados::cls::lock::get_lock_info_start(&op, RBD_LOCK_NAME);
 
       bufferlist outbl;
       int r = ioctx->operate(oid, &op, &outbl);
@@ -71,13 +84,28 @@ namespace librbd {
       try {
 	bufferlist::iterator iter = outbl.begin();
 	uint8_t order;
+	// get_size
 	::decode(order, iter);
 	::decode(*size, iter);
+	// get_features
 	::decode(*features, iter);
 	::decode(*incompatible_features, iter);
+	// get_snapcontext
 	::decode(*snapc, iter);
-	::decode(*lockers, iter);
-	::decode(*exclusive_lock, iter);
+	// get_parent
+	::decode(parent->spec.pool_id, iter);
+	::decode(parent->spec.image_id, iter);
+	::decode(parent->spec.snap_id, iter);
+	::decode(parent->overlap, iter);
+
+	// get_lock_info
+	ClsLockType lock_type;
+	r = rados::cls::lock::get_lock_info_finish(&iter, lockers, &lock_type,
+						   lock_tag);
+	if (r < 0)
+	  return r;
+
+	*exclusive_lock = (lock_type == LOCK_EXCLUSIVE);
       } catch (const buffer::error &err) {
 	return -EBADMSG;
       }
@@ -167,8 +195,7 @@ namespace librbd {
     }
 
     int get_parent(librados::IoCtx *ioctx, const std::string &oid,
-		   snapid_t snap_id, int64_t *parent_pool,
-		   string *parent_image, snapid_t *parent_snap_id,
+		   snapid_t snap_id, parent_spec *pspec, 
 		   uint64_t *parent_overlap)
     {
       bufferlist inbl, outbl;
@@ -180,9 +207,9 @@ namespace librbd {
 
       try {
 	bufferlist::iterator iter = outbl.begin();
-	::decode(*parent_pool, iter);
-	::decode(*parent_image, iter);
-	::decode(*parent_snap_id, iter);
+	::decode(pspec->pool_id, iter);
+	::decode(pspec->image_id, iter);
+	::decode(pspec->snap_id, iter);
 	::decode(*parent_overlap, iter);
       } catch (const buffer::error &err) {
 	return -EBADMSG;
@@ -192,13 +219,12 @@ namespace librbd {
     }
 
     int set_parent(librados::IoCtx *ioctx, const std::string &oid,
-		   int64_t parent_pool, const string& parent_image,
-		   snapid_t parent_snap_id, uint64_t parent_overlap)
+		   parent_spec pspec, uint64_t parent_overlap)
     {
       bufferlist inbl, outbl;
-      ::encode(parent_pool, inbl);
-      ::encode(parent_image, inbl);
-      ::encode(parent_snap_id, inbl);
+      ::encode(pspec.pool_id, inbl);
+      ::encode(pspec.image_id, inbl);
+      ::encode(pspec.snap_id, inbl);
       ::encode(parent_overlap, inbl);
 
       return ioctx->exec(oid, "rbd", "set_parent", inbl, outbl);
@@ -210,6 +236,50 @@ namespace librbd {
       return ioctx->exec(oid, "rbd", "remove_parent", inbl, outbl);
     }
 
+    int add_child(librados::IoCtx *ioctx, const std::string &oid,
+		  parent_spec pspec, const std::string &c_imageid)
+    {
+      bufferlist in, out;
+      ::encode(pspec.pool_id, in);
+      ::encode(pspec.image_id, in);
+      ::encode(pspec.snap_id, in);
+      ::encode(c_imageid, in);
+
+      return ioctx->exec(oid, "rbd", "add_child", in, out);
+    }
+
+    int remove_child(librados::IoCtx *ioctx, const std::string &oid,
+		     parent_spec pspec, const std::string &c_imageid)
+    {
+      bufferlist in, out;
+      ::encode(pspec.pool_id, in);
+      ::encode(pspec.image_id, in);
+      ::encode(pspec.snap_id, in);
+      ::encode(c_imageid, in);
+
+      return ioctx->exec(oid, "rbd", "remove_child", in, out);
+    }
+
+    int get_children(librados::IoCtx *ioctx, const std::string &oid,
+		     parent_spec pspec, set<string>& children)
+    {
+      bufferlist in, out;
+      ::encode(pspec.pool_id, in);
+      ::encode(pspec.image_id, in);
+      ::encode(pspec.snap_id, in);
+
+      int r = ioctx->exec(oid, "rbd", "get_children", in, out);
+      if (r < 0)
+	return r;
+      bufferlist::iterator it = out.begin();
+      try {
+	::decode(children, it);
+      } catch (const buffer::error &err) {
+	return -EBADMSG;
+      }
+      return 0;
+    }
+
     int snapshot_add(librados::IoCtx *ioctx, const std::string &oid,
 		     snapid_t snap_id, const std::string &snap_name)
     {
@@ -255,7 +325,9 @@ namespace librbd {
 		      const std::vector<snapid_t> &ids,
 		      std::vector<string> *names,
 		      std::vector<uint64_t> *sizes,
-		      std::vector<uint64_t> *features)
+		      std::vector<uint64_t> *features,
+		      std::vector<parent_info> *parents,
+		      std::vector<uint8_t> *protection_statuses)
     {
       names->clear();
       names->resize(ids.size());
@@ -263,17 +335,26 @@ namespace librbd {
       sizes->resize(ids.size());
       features->clear();
       features->resize(ids.size());
+      parents->clear();
+      parents->resize(ids.size());
+      protection_statuses->clear();
+      protection_statuses->resize(ids.size());
+
       librados::ObjectReadOperation op;
       for (vector<snapid_t>::const_iterator it = ids.begin();
 	   it != ids.end(); ++it) {
-	bufferlist bl1, bl2, bl3;
 	snapid_t snap_id = it->val;
+	bufferlist bl1, bl2, bl3, bl4, bl5;
 	::encode(snap_id, bl1);
 	op.exec("rbd", "get_snapshot_name", bl1);
 	::encode(snap_id, bl2);
 	op.exec("rbd", "get_size", bl2);
 	::encode(snap_id, bl3);
 	op.exec("rbd", "get_features", bl3);
+	::encode(snap_id, bl4);
+	op.exec("rbd", "get_parent", bl4);
+	::encode(snap_id, bl5);
+	op.exec("rbd", "get_protection_status", bl5);
       }
 
       bufferlist outbl;
@@ -286,11 +367,21 @@ namespace librbd {
 	for (size_t i = 0; i < ids.size(); ++i) {
 	  uint8_t order;
 	  uint64_t incompat_features;
+	  // get_snapshot_name
 	  ::decode((*names)[i], iter);
+	  // get_size
 	  ::decode(order, iter);
 	  ::decode((*sizes)[i], iter);
+	  // get_features
 	  ::decode((*features)[i], iter);
 	  ::decode(incompat_features, iter);
+	  // get_parent
+	  ::decode((*parents)[i].spec.pool_id, iter);
+	  ::decode((*parents)[i].spec.image_id, iter);
+	  ::decode((*parents)[i].spec.snap_id, iter);
+	  ::decode((*parents)[i].overlap, iter);
+	  // get_protection_status
+	  ::decode((*protection_statuses)[i], iter);
 	}
       } catch (const buffer::error &err) {
 	return -EBADMSG;
@@ -350,56 +441,39 @@ namespace librbd {
       return 0;
     }
 
-    int list_locks(librados::IoCtx *ioctx, const std::string &oid,
-                   std::set<std::pair<std::string, std::string> > &locks,
-                   bool &exclusive)
+    int copyup(librados::IoCtx *ioctx, const std::string &oid,
+	       bufferlist data) {
+      bufferlist out;
+      return ioctx->exec(oid, "rbd", "copyup", data, out);
+    }
+
+    int get_protection_status(librados::IoCtx *ioctx, const std::string &oid,
+			      snapid_t snap_id, uint8_t *protection_status)
     {
       bufferlist in, out;
-      int r = ioctx->exec(oid, "rbd", "list_locks", in, out);
-      if (r < 0) {
-        return r;
+      ::encode(snap_id.val, in);
+
+      int r = ioctx->exec(oid, "rbd", "get_protection_status", in, out);
+      if (r < 0)
+	return r;
+
+      try {
+	bufferlist::iterator iter = out.begin();
+	::decode(*protection_status, iter);
+      } catch (const buffer::error &err) {
+	return -EBADMSG;
       }
 
-      bufferlist::iterator iter = out.begin();
-      try {
-        ::decode(locks, iter);
-        ::decode(exclusive, iter);
-      } catch (const buffer::error &err) {
-        return -EBADMSG;
-      }
       return 0;
     }
 
-    int lock_image_exclusive(librados::IoCtx *ioctx, const std::string &oid,
-                             const std::string &cookie)
+    int set_protection_status(librados::IoCtx *ioctx, const std::string &oid,
+			      snapid_t snap_id, uint8_t protection_status)
     {
       bufferlist in, out;
-      ::encode(cookie, in);
-      return ioctx->exec(oid, "rbd", "lock_exclusive", in, out);
-    }
-
-    int lock_image_shared(librados::IoCtx *ioctx, const std::string &oid,
-                          const std::string &cookie)
-    {
-      bufferlist in, out;
-      ::encode(cookie, in);
-      return ioctx->exec(oid, "rbd", "lock_shared", in, out);
-    }
-
-    int unlock_image(librados::IoCtx *ioctx, const std::string& oid,
-                         const std::string &cookie)
-    {
-      bufferlist in, out;
-      ::encode(cookie, in);
-      return ioctx->exec(oid, "rbd", "unlock_image", in, out);
-    }
-    int break_lock(librados::IoCtx *ioctx, const std::string& oid,
-                   const std::string &locker, const std::string &cookie)
-    {
-      bufferlist in, out;
-      ::encode(locker, in);
-      ::encode(cookie, in);
-      return ioctx->exec(oid, "rbd", "break_lock", in, out);
+      ::encode(snap_id, in);
+      ::encode(protection_status, in);
+      return ioctx->exec(oid, "rbd", "set_protection_status", in, out);
     }
 
     /************************ rbd_id object methods ************************/
