@@ -32,7 +32,7 @@
 
 #include "auth/Auth.h"
 #include "auth/KeyRing.h"
-#include "auth/AuthSupported.h"
+#include "auth/AuthMethodList.h"
 
 #include "include/str_list.h"
 #include "include/addr_parsing.h"
@@ -40,7 +40,7 @@
 #include "common/config.h"
 
 
-#define DOUT_SUBSYS monc
+#define dout_subsys ceph_subsys_monc
 #undef dout_prefix
 #define dout_prefix *_dout << "monclient" << (hunting ? "(hunting)":"") << ": "
 
@@ -53,6 +53,7 @@ MonClient::MonClient(CephContext *cct_) :
   timer(cct_, monc_lock), finisher(cct_),
   initialized(false),
   log_client(NULL),
+  more_log_pending(false),
   auth_supported(NULL),
   hunting(true),
   want_monmap(true),
@@ -73,130 +74,10 @@ MonClient::~MonClient()
   delete rotating_secrets;
 }
 
-/*
- * build an initial monmap with any known monitor
- * addresses.
- */
-int MonClient::build_initial_monmap(CephContext *cct, MonMap &monmap)
-{
-  const md_config_t *conf = cct->_conf;
-  // file?
-  if (!conf->monmap.empty()) {
-    int r;
-    try {
-      r = monmap.read(conf->monmap.c_str());
-    }
-    catch (const buffer::error &e) {
-      r = -EINVAL;
-    }
-    if (r >= 0)
-      return 0;
-    cerr << "unable to read/decode monmap from " << conf->monmap
-	 << ": " << cpp_strerror(-r) << std::endl;
-    return r;
-  }
-
-  // fsid from conf?
-  if (cct->_conf->fsid.length()) {
-    if (!monmap.fsid.parse(cct->_conf->fsid.c_str())) {
-      cerr << " failed to parse fsid '" << cct->_conf->fsid << "'" << std::endl;
-    }
-  }
-
-  // -m foo?
-  if (!conf->mon_host.empty()) {
-    vector<entity_addr_t> addrs;
-    if (parse_ip_port_vec(conf->mon_host.c_str(), addrs)) {
-      for (unsigned i=0; i<addrs.size(); i++) {
-	char n[2];
-	n[0] = 'a' + i;
-	n[1] = 0;
-	if (addrs[i].get_port() == 0)
-	  addrs[i].set_port(CEPH_MON_PORT);
-	string name = "noname-";
-	name += n;
-	monmap.add(name, addrs[i]);
-      }
-      return 0;
-    } else { //maybe they passed us a DNS-resolvable name
-      char *hosts = NULL;
-      hosts = resolve_addrs(conf->mon_host.c_str());
-      if (!hosts)
-        return -EINVAL;
-      bool success = parse_ip_port_vec(hosts, addrs);
-      free(hosts);
-      if (success) {
-        for (unsigned i=0; i<addrs.size(); i++) {
-          char n[2];
-          n[0] = 'a' + i;
-          n[1] = 0;
-          if (addrs[i].get_port() == 0)
-            addrs[i].set_port(CEPH_MON_PORT);
-          monmap.add(n, addrs[i]);
-        }
-        return 0;
-      } else cerr << "couldn't parse_ip_port_vec on " << hosts << std::endl;
-    }
-    cerr << "unable to parse addrs in '" << conf->mon_host << "'" << std::endl;
-  }
-
-  // What monitors are in the config file?
-  std::vector <std::string> sections;
-  int ret = conf->get_all_sections(sections);
-  if (ret) {
-    cerr << "Unable to find any monitors in the configuration "
-         << "file, because there was an error listing the sections. error "
-	 << ret << std::endl;
-    return -ENOENT;
-  }
-  std::vector <std::string> mon_names;
-  for (std::vector <std::string>::const_iterator s = sections.begin();
-       s != sections.end(); ++s) {
-    if ((s->substr(0, 4) == "mon.") && (s->size() > 4)) {
-      mon_names.push_back(s->substr(4));
-    }
-  }
-
-  // Find an address for each monitor in the config file.
-  for (std::vector <std::string>::const_iterator m = mon_names.begin();
-       m != mon_names.end(); ++m) {
-    std::vector <std::string> sections;
-    std::string m_name("mon");
-    m_name += ".";
-    m_name += *m;
-    sections.push_back(m_name);
-    sections.push_back("mon");
-    sections.push_back("global");
-    std::string val;
-    int res = conf->get_val_from_conf_file(sections, "mon addr", val, true);
-    if (res) {
-      cerr << "failed to get an address for mon." << *m << ": error "
-	   << res << std::endl;
-      continue;
-    }
-    entity_addr_t addr;
-    if (!addr.parse(val.c_str())) {
-      cerr << "unable to parse address for mon." << *m
-	   << ": addr='" << val << "'" << std::endl;
-      continue;
-    }
-    if (addr.get_port() == 0)
-      addr.set_port(CEPH_MON_PORT);    
-    monmap.add(m->c_str(), addr);
-  }
-
-  if (monmap.size() == 0) {
-    cerr << "unable to find any monitors in conf. "
-	 << "please specify monitors via -m monaddr or -c ceph.conf" << std::endl;
-    return -ENOENT;
-  }
-  return 0;
-}
-
 int MonClient::build_initial_monmap()
 {
   ldout(cct, 10) << "build_initial_monmap" << dendl;
-  return build_initial_monmap(cct, monmap);
+  return monmap.build_initial(cct, cerr);
 }
 
 int MonClient::get_monmap()
@@ -225,6 +106,7 @@ int MonClient::get_monmap_privately()
   if (!messenger) {
     messenger = smessenger = new SimpleMessenger(cct,
                                                  entity_name_t::CLIENT(-1),
+						 "temp_mon_client",
                                                  getpid());
     messenger->add_dispatcher_head(this);
     smessenger->start();
@@ -288,6 +170,7 @@ bool MonClient::ms_dispatch(Message *m)
   case CEPH_MSG_AUTH_REPLY:
   case CEPH_MSG_MON_SUBSCRIBE_ACK:
   case CEPH_MSG_MON_GET_VERSION_REPLY:
+  case MSG_LOGACK:
     break;
   default:
     return false;
@@ -314,10 +197,31 @@ bool MonClient::ms_dispatch(Message *m)
     break;
   case CEPH_MSG_MON_GET_VERSION_REPLY:
     handle_get_version_reply((MMonGetVersionReply*)m);
+    break;
+  case MSG_LOGACK:
+    if (log_client) {
+      log_client->handle_log_ack((MLogAck*)m);
+      if (more_log_pending) {
+	send_log();
+      }
+    } else {
+      m->put();
+    }
+    break;
   }
   return true;
 }
 
+void MonClient::send_log()
+{
+  if (log_client) {
+    Message *lm = log_client->get_mon_log_message();
+    if (lm)
+      _send_mon_message(lm);
+    more_log_pending = log_client->are_pending();
+  }
+}
+
 void MonClient::handle_monmap(MMonMap *m)
 {
   ldout(cct, 10) << "handle_monmap " << *m << dendl;
@@ -351,19 +255,37 @@ void MonClient::handle_monmap(MMonMap *m)
 
 int MonClient::init()
 {
-  ldout(cct, 10) << "init auth_supported " << cct->_conf->auth_supported << dendl;
+  ldout(cct, 10) << "init" << dendl;
 
   messenger->add_dispatcher_head(this);
 
-  int r = KeyRing::from_ceph_context(cct, &keyring);
+  entity_name = cct->_conf->name;
+
+  // keyring
+  keyring = new KeyRing;
+  int r = keyring->from_ceph_context(cct);
+  if (r == -ENOENT) {
+    // do we care?
+    string method;
+    if (entity_name.get_type() == CEPH_ENTITY_TYPE_MDS ||
+	entity_name.get_type() == CEPH_ENTITY_TYPE_OSD)
+      method = cct->_conf->auth_cluster_required;
+    else
+      method = cct->_conf->auth_client_required;
+    if (method.length() == 0)
+      method = cct->_conf->auth_supported;
+    AuthMethodList supported(cct, method);
+    if (!supported.is_supported_auth(CEPH_AUTH_CEPHX)) {
+      ldout(cct, 2) << "cephx auth is not supported, ignoring absence of keyring" << dendl;
+      r = 0;
+    }
+  }
   if (r < 0) {
     lderr(cct) << "failed to open keyring: " << cpp_strerror(r) << dendl;
     return r;
   }
   rotating_secrets = new RotatingKeyRing(cct, cct->get_module_type(), keyring);
 
-  entity_name = cct->_conf->name;
-  
   Mutex::Locker l(monc_lock);
   timer.init();
   finisher.start();
@@ -372,7 +294,14 @@ int MonClient::init()
   // seed rng so we choose a different monitor each time
   srand(getpid());
 
-  auth_supported = new AuthSupported(cct);
+  string method;
+  if (entity_name.get_type() == CEPH_ENTITY_TYPE_OSD ||
+      entity_name.get_type() == CEPH_ENTITY_TYPE_MDS ||
+      entity_name.get_type() == CEPH_ENTITY_TYPE_MON)
+    method = cct->_conf->auth_cluster_required;
+  else
+    method = cct->_conf->auth_client_required;
+  auth_supported = new AuthMethodList(cct, method.length() ? method : cct->_conf->auth_supported);
   ldout(cct, 10) << "auth_supported " << auth_supported->get_supported_set() << dendl;
 
   initialized = true;
@@ -487,9 +416,7 @@ void MonClient::handle_auth(MAuthReply *m)
 
       if (log_client) {
 	log_client->reset_session();
-	Message *lm = log_client->get_mon_log_message();
-	if (lm)
-	  _send_mon_message(lm);
+	send_log();
       }
     }
   
@@ -519,6 +446,7 @@ void MonClient::_pick_new_mon()
 {
   assert(monc_lock.is_locked());
 
+  assert(monmap.size() > 0);
   if (!cur_mon.empty() && monmap.size() > 1) {
     // pick a _different_ mon
     cur_mon = monmap.pick_random_mon_not(cur_mon);
@@ -553,7 +481,7 @@ void MonClient::_reopen_session()
 
   // throw out version check requests
   while (!version_requests.empty()) {
-    finisher.queue(version_requests.begin()->second->context, -1);
+    finisher.queue(version_requests.begin()->second->context, -EAGAIN);
     version_requests.erase(version_requests.begin());
   }
 
@@ -617,16 +545,17 @@ void MonClient::tick()
   } else if (!cur_mon.empty()) {
     // just renew as needed
     utime_t now = ceph_clock_now(cct);
+    ldout(cct, 10) << "renew subs? (now: " << now 
+		   << "; renew after: " << sub_renew_after << ") -- " 
+		   << (now > sub_renew_after ? "yes" : "no") 
+		   << dendl;
     if (now > sub_renew_after)
       _renew_subs();
 
     messenger->send_keepalive(cur_con);
    
-    if (state == MC_STATE_HAVE_SESSION &&
-	log_client) {
-      Message *m = log_client->get_mon_log_message();
-      if (m)
-	_send_mon_message(m);
+    if (state == MC_STATE_HAVE_SESSION) {
+      send_log();
     }
   }
 
@@ -644,7 +573,6 @@ void MonClient::schedule_tick()
     timer.add_event_after(cct->_conf->mon_client_ping_interval, new C_Tick(this));
 }
 
-
 // ---------
 
 void MonClient::_renew_subs()
@@ -766,9 +694,9 @@ struct C_IsLatestMap : public Context {
   Context *onfinish;
   version_t newest;
   version_t have;
-  C_IsLatestMap(Context *f, version_t h) : onfinish(f), have(h) {}
+  C_IsLatestMap(Context *f, version_t h) : onfinish(f), newest(0), have(h) {}
   void finish(int r) {
-    onfinish->complete(have != newest);
+    onfinish->complete(r == 0 ? (have != newest) : r);
   }
 };
 
