@@ -35,6 +35,79 @@ class MOSDBoot;
 class MMonCommand;
 class MPoolSnap;
 class MOSDMap;
+class MOSDFailure;
+
+/// information about a particular peer's failure reports for one osd
+struct failure_reporter_t {
+  int num_reports;          ///< reports from this reporter
+  utime_t failed_since;     ///< when they think it failed
+  MOSDFailure *msg;         ///< most recent failure message
+
+  failure_reporter_t() : num_reports(0), msg(NULL) {}
+  failure_reporter_t(utime_t s) : num_reports(1), failed_since(s), msg(NULL) {}
+};
+
+/// information about all failure reports for one osd
+struct failure_info_t {
+  map<int, failure_reporter_t> reporters;  ///< reporter -> # reports
+  utime_t max_failed_since;                ///< most recent failed_since
+  int num_reports;
+
+  failure_info_t() : num_reports(0) {}
+
+  utime_t get_failed_since() {
+    if (max_failed_since == utime_t() && reporters.size()) {
+      // the old max must have canceled; recalculate.
+      for (map<int, failure_reporter_t>::iterator p = reporters.begin();
+	   p != reporters.end();
+	   ++p)
+	if (p->second.failed_since > max_failed_since)
+	  max_failed_since = p->second.failed_since;
+    }
+    return max_failed_since;
+  }
+
+  // set the message for the latest report.  return any old message we had,
+  // if any, so we can discard it.
+  MOSDFailure *add_report(int who, utime_t failed_since, MOSDFailure *msg) {
+    map<int, failure_reporter_t>::iterator p = reporters.find(who);
+    if (p == reporters.end()) {
+      if (max_failed_since == utime_t())
+	max_failed_since = failed_since;
+      else if (max_failed_since < failed_since)
+	max_failed_since = failed_since;
+      p = reporters.insert(map<int, failure_reporter_t>::value_type(who, failure_reporter_t(failed_since))).first;
+    } else {
+      p->second.num_reports++;
+    }
+    num_reports++;
+
+    MOSDFailure *ret = p->second.msg;
+    p->second.msg = msg;
+    return ret;
+  }
+
+  void take_report_messages(list<MOSDFailure*>& ls) {
+    for (map<int, failure_reporter_t>::iterator p = reporters.begin();
+	 p != reporters.end();
+	 ++p) {
+      if (p->second.msg) {
+	ls.push_back(p->second.msg);
+	p->second.msg = NULL;
+      }
+    }
+  }
+
+  void cancel_report(int who) {
+    map<int, failure_reporter_t>::iterator p = reporters.find(who);
+    if (p == reporters.end())
+      return;
+    num_reports -= p->second.num_reports;
+    reporters.erase(p);
+    if (reporters.empty())
+      max_failed_since = utime_t();
+  }
+};
 
 class OSDMonitor : public PaxosService {
 public:
@@ -45,10 +118,19 @@ private:
 
   // [leader]
   OSDMap::Incremental pending_inc;
-  multimap<int, pair<int, int> > failed_notes; // <failed_osd, <reporter, #reports> >
+  map<int, failure_info_t> failure_info;
   map<int,utime_t>    down_pending_out;  // osd down -> out
 
   map<int,double> osd_weight;
+
+  void check_failures(utime_t now);
+  bool check_failure(utime_t now, int target_osd, failure_info_t& fi);
+
+  // map thrashing
+  int thrash_map;
+  int thrash_last_up_osd;
+  bool thrash();
+
   // svc
 public:  
   void create_initial();
@@ -58,6 +140,8 @@ private:
   void encode_pending(bufferlist &bl);
   void on_active();
 
+  void update_msgr_features();
+
   void share_map_with_random_osd();
 
   void update_logger();
@@ -67,6 +151,11 @@ private:
   bool prepare_update(PaxosServiceMessage *m);
   bool should_propose(double &delay);
 
+  bool can_mark_down(int o);
+  bool can_mark_up(int o);
+  bool can_mark_out(int o);
+  bool can_mark_in(int o);
+
   // ...
   void send_to_waiting();     // send current map to waiters.
   MOSDMap *build_latest_full();
@@ -80,7 +169,7 @@ private:
  
   bool preprocess_failure(class MOSDFailure *m);
   bool prepare_failure(class MOSDFailure *m);
-  void _reported_failure(MOSDFailure *m);
+  void _reported_failure(list<MOSDFailure*>& m);
 
   bool preprocess_boot(class MOSDBoot *m);
   bool prepare_boot(class MOSDBoot *m);
@@ -94,6 +183,7 @@ private:
   bool prepare_pgtemp(class MOSDPGTemp *m);
 
   int _prepare_remove_pool(uint64_t pool);
+  int _prepare_rename_pool(uint64_t pool, string newname);
 
   bool preprocess_pool_op ( class MPoolOp *m);
   bool preprocess_pool_op_create ( class MPoolOp *m);
@@ -104,6 +194,9 @@ private:
   int prepare_new_pool(string& name, uint64_t auid, int crush_rule,
                        unsigned pg_num, unsigned pgp_num);
   int prepare_new_pool(MPoolOp *m);
+  
+  bool prepare_set_flag(MMonCommand *m, int flag);
+  bool prepare_unset_flag(MMonCommand *m, int flag);
 
   void _pool_op_reply(MPoolOp *m, int ret, epoch_t epoch, bufferlist *blp=NULL);
 
@@ -132,14 +225,20 @@ private:
   };
   struct C_Reported : public Context {
     OSDMonitor *cmon;
-    MOSDFailure *m;
-    C_Reported(OSDMonitor *cm, MOSDFailure *m_) : 
-      cmon(cm), m(m_) {}
+    list<MOSDFailure*> msgs;
+    C_Reported(OSDMonitor *cm, list<MOSDFailure*>& m_)
+      : cmon(cm) {
+      msgs.swap(m_);
+    }
     void finish(int r) {
       if (r >= 0)
-	cmon->_reported_failure(m);
-      else
-	cmon->dispatch((PaxosServiceMessage*)m);
+	cmon->_reported_failure(msgs);
+      else {
+	while (!msgs.empty()) {
+	  cmon->dispatch((PaxosServiceMessage*)msgs.front());
+	  msgs.pop_front();
+	}
+      }
     }
   };
   struct C_PoolOp : public Context {
@@ -163,13 +262,16 @@ private:
 
   void tick();  // check state, take actions
 
+  int parse_osd_id(const char *s, stringstream *pss);
+  void parse_loc_map(const vector<string>& args, int start, map<string,string> *ploc);
+
   void get_health(list<pair<health_status_t,string> >& summary,
 		  list<pair<health_status_t,string> > *detail) const;
   bool preprocess_command(MMonCommand *m);
   bool prepare_command(MMonCommand *m);
 
   void handle_osd_timeouts(const utime_t &now,
-			   const std::map<int,utime_t> &last_osd_report);
+			   std::map<int,utime_t> &last_osd_report);
   void mark_all_down();
 
   void send_latest(PaxosServiceMessage *m, epoch_t start=0);
@@ -179,6 +281,8 @@ private:
 
   epoch_t blacklist(entity_addr_t a, utime_t until);
 
+  void dump_info(Formatter *f);
+
   void check_subs();
   void check_sub(Subscription *sub);
 
