@@ -97,6 +97,7 @@ extern int cls_register_cxx_method(cls_handle_t hclass, const char *method, int
 				   cls_method_cxx_call_t class_call, cls_method_handle_t *handle);
 
 extern int cls_cxx_create(cls_method_context_t hctx, bool exclusive);
+extern int cls_cxx_remove(cls_method_context_t hctx);
 extern int cls_cxx_stat(cls_method_context_t hctx, uint64_t *size, time_t *mtime);
 extern int cls_cxx_read(cls_method_context_t hctx, int ofs, int len, bufferlist *bl);
 extern int cls_cxx_write(cls_method_context_t hctx, int ofs, int len, bufferlist *bl);
@@ -126,7 +127,7 @@ extern int cls_cxx_map_get_val(cls_method_context_t hctx,
 extern int cls_cxx_map_set_val(cls_method_context_t hctx,
                                const string &key, bufferlist *inbl);
 extern int cls_cxx_map_set_vals(cls_method_context_t hctx,
-                                std::map<string, bufferlist> *map);
+                                const std::map<string, bufferlist> *map);
 extern int cls_cxx_map_write_header(cls_method_context_t hctx, bufferlist *inbl);
 extern int cls_cxx_map_remove_key(cls_method_context_t hctx, const string &key);
 extern int cls_cxx_map_update(cls_method_context_t hctx, bufferlist *inbl);
