@@ -26,6 +26,7 @@
 #include "common/Timer.h"
 #include "common/WorkQueue.h"
 #include "common/LogClient.h"
+#include "common/AsyncReserver.h"
 
 #include "os/ObjectStore.h"
 #include "OSDCap.h"
@@ -93,6 +94,9 @@ enum {
   l_osd_push,
   l_osd_push_outb,
 
+  l_osd_push_in,
+  l_osd_push_inb,
+
   l_osd_rop,
 
   l_osd_loadavg,
@@ -184,7 +188,7 @@ public:
     Mutex::Locker l(publish_lock);
     return superblock;
   }
-  void publish_superblock(OSDSuperblock block) {
+  void publish_superblock(const OSDSuperblock &block) {
     Mutex::Locker l(publish_lock);
     superblock = block;
   }
@@ -198,7 +202,6 @@ public:
     osdmap = map;
   }
 
-
   int get_nodeid() const { return whoami; }
 
   // -- scrub scheduling --
@@ -244,6 +247,10 @@ public:
   SafeTimer watch_timer;
   Watch *watch;
 
+  // -- Backfill Request Scheduling --
+  Mutex backfill_request_lock;
+  SafeTimer backfill_request_timer;
+
   // -- tids --
   // for ops i issue
   tid_t last_tid;
@@ -256,6 +263,11 @@ public:
     return t;
   }
 
+  // -- backfill_reservation --
+  Finisher reserver_finisher;
+  AsyncReserver<pg_t> local_reserver;
+  AsyncReserver<pg_t> remote_reserver;
+
   // -- pg_temp --
   Mutex pg_temp_lock;
   map<pg_t, vector<int> > pg_temp_wanted;
@@ -282,7 +294,6 @@ public:
   SimpleLRU<epoch_t, bufferlist> map_bl_cache;
   SimpleLRU<epoch_t, bufferlist> map_bl_inc_cache;
 
-
   OSDMapRef get_map(epoch_t e);
   OSDMapRef add_map(OSDMap *o) {
     Mutex::Locker l(map_cache_lock);
@@ -317,6 +328,9 @@ public:
   void pg_stat_queue_enqueue(PG *pg);
   void pg_stat_queue_dequeue(PG *pg);
 
+  void init();
+  void shutdown();
+
   OSDService(OSD *osd);
 };
 class OSD : public Dispatcher {
@@ -325,7 +339,8 @@ protected:
   Mutex osd_lock;			// global lock
   SafeTimer timer;    // safe timer (osd_lock)
 
-  AuthAuthorizeHandlerRegistry *authorize_handler_registry;
+  AuthAuthorizeHandlerRegistry *authorize_handler_cluster_registry;
+  AuthAuthorizeHandlerRegistry *authorize_handler_service_registry;
 
   Messenger   *cluster_messenger;
   Messenger   *client_messenger;
@@ -355,6 +370,8 @@ protected:
   void _dispatch(Message *m);
   void dispatch_op(OpRequestRef op);
 
+  void check_osdmap_features();
+
 public:
   ClassHandler  *class_handler;
   int get_nodeid() { return whoami; }
@@ -421,6 +438,8 @@ private:
   ThreadPool disk_tp;
   ThreadPool command_tp;
 
+  bool paused_recovery;
+
   // -- sessions --
 public:
   struct Session : public RefCountedObject {
@@ -447,12 +466,18 @@ private:
   // -- heartbeat --
   /// information about a heartbeat peer
   struct HeartbeatInfo {
+    entity_inst_t inst; ///< peer
     Connection *con;    ///< peer connection
     utime_t first_tx;   ///< time we sent our first ping request
     utime_t last_tx;    ///< last time we sent a ping request
     utime_t last_rx;    ///< last time we got a ping reply
     epoch_t epoch;      ///< most recent epoch we wanted this peer
   };
+  /// state attached to outgoing heartbeat connections
+  struct HeartbeatSession : public RefCountedObject {
+    int peer;
+    HeartbeatSession(int p) : peer(p) {}
+  };
   Mutex heartbeat_lock;
   map<int, int> debug_heartbeat_drops_remaining;
   Cond heartbeat_cond;
@@ -464,6 +489,7 @@ private:
   Messenger *hbclient_messenger, *hbserver_messenger;
   
   void _add_heartbeat_peer(int p);
+  bool heartbeat_reset(Connection *con);
   void maybe_update_heartbeat_peers();
   void reset_heartbeat_peers();
   void heartbeat();
@@ -488,7 +514,9 @@ public:
     bool ms_dispatch(Message *m) {
       return osd->heartbeat_dispatch(m);
     };
-    bool ms_handle_reset(Connection *con) { return false; }
+    bool ms_handle_reset(Connection *con) {
+      return osd->heartbeat_reset(con);
+    }
     void ms_handle_remote_reset(Connection *con) {}
   public:
     OSD *osd;
@@ -719,8 +747,9 @@ protected:
   PGPool _get_pool(int id, OSDMapRef createmap);
 
   bool  _have_pg(pg_t pgid);
-  PG   *_lookup_lock_pg(pg_t pgid);
   PG   *_lookup_lock_pg_with_map_lock_held(pg_t pgid);
+  PG   *_lookup_lock_pg(pg_t pgid);
+  PG   *_lookup_pg(pg_t pgid);
   PG   *_open_lock_pg(OSDMapRef createmap,
 		      pg_t pg, bool no_lockdep_check=false,
 		      bool hold_map_lock=false);
@@ -731,6 +760,7 @@ protected:
 			pg_history_t history,
 			pg_interval_map_t& pi,
 			ObjectStore::Transaction& t);
+  PG   *_lookup_qlock_pg(pg_t pgid);
 
   PG *lookup_lock_raw_pg(pg_t pgid);
 
@@ -813,15 +843,12 @@ protected:
   void send_alive();
 
   // -- failures --
-  set<int> failure_queue;
+  map<int,utime_t> failure_queue;
   map<int,entity_inst_t> failure_pending;
 
 
-  void queue_failure(int n) {
-    failure_queue.insert(n);
-  }
   void send_failures();
-  void send_still_alive(epoch_t epoch, entity_inst_t i);
+  void send_still_alive(epoch_t epoch, const entity_inst_t &i);
 
   // -- pg stats --
   Mutex pg_stat_queue_lock;
@@ -889,6 +916,7 @@ protected:
   void handle_pg_scan(OpRequestRef op);
 
   void handle_pg_backfill(OpRequestRef op);
+  void handle_pg_backfill_reserve(OpRequestRef op);
 
   void handle_pg_remove(OpRequestRef op);
   void _remove_pg(PG *pg);
@@ -1315,8 +1343,6 @@ public:
 
   /// check if we can throw out op from a disconnected client
   static bool op_is_discardable(class MOSDOp *m);
-  /// check if op has sufficient caps
-  bool op_has_sufficient_caps(PG *pg, class MOSDOp *m);
   /// check if op should be (re)queued for processing
 public:
   void force_remount();
