@@ -82,6 +82,52 @@ ostream& operator<<(ostream& out, const osd_info_t& info)
   return out;
 }
 
+// ----------------------------------
+// osd_xinfo_t
+
+void osd_xinfo_t::dump(Formatter *f) const
+{
+  f->dump_stream("down_stamp") << down_stamp;
+  f->dump_float("laggy_probability", laggy_probability);
+  f->dump_int("laggy_interval", laggy_interval);
+}
+
+void osd_xinfo_t::encode(bufferlist& bl) const
+{
+  ENCODE_START(1, 1, bl);
+  ::encode(down_stamp, bl);
+  __u32 lp = laggy_probability * 0xfffffffful;
+  ::encode(lp, bl);
+  ::encode(laggy_interval, bl);
+  ENCODE_FINISH(bl);
+}
+
+void osd_xinfo_t::decode(bufferlist::iterator& bl)
+{
+  DECODE_START(1, bl);
+  ::decode(down_stamp, bl);
+  __u32 lp;
+  ::decode(lp, bl);
+  laggy_probability = (float)lp / (float)0xffffffff;
+  ::decode(laggy_interval, bl);
+  DECODE_FINISH(bl);
+}
+
+void osd_xinfo_t::generate_test_instances(list<osd_xinfo_t*>& o)
+{
+  o.push_back(new osd_xinfo_t);
+  o.push_back(new osd_xinfo_t);
+  o.back()->down_stamp = utime_t(2, 3);
+  o.back()->laggy_probability = .123;
+  o.back()->laggy_interval = 123456;
+}
+
+ostream& operator<<(ostream& out, const osd_xinfo_t& xi)
+{
+  return out << "down_stamp " << xi.down_stamp
+	     << " laggy_probability " << xi.laggy_probability
+	     << " laggy_interval " << xi.laggy_interval;
+}
 
 // ----------------------------------
 // OSDMap::Incremental
@@ -208,7 +254,7 @@ void OSDMap::Incremental::encode(bufferlist& bl, uint64_t features) const
   ::encode(new_pg_temp, bl);
 
   // extended
-  __u16 ev = 8;
+  __u16 ev = 9;
   ::encode(ev, bl);
   ::encode(new_hb_up, bl);
   ::encode(new_up_thru, bl);
@@ -219,6 +265,7 @@ void OSDMap::Incremental::encode(bufferlist& bl, uint64_t features) const
   ::encode(new_up_internal, bl);
   ::encode(cluster_snapshot, bl);
   ::encode(new_uuid, bl);
+  ::encode(new_xinfo, bl);
 }
 
 void OSDMap::Incremental::decode(bufferlist::iterator &p)
@@ -308,6 +355,8 @@ void OSDMap::Incremental::decode(bufferlist::iterator &p)
     ::decode(cluster_snapshot, p);
   if (ev >= 8)
     ::decode(new_uuid, p);
+  if (ev >= 9)
+    ::decode(new_xinfo, p);
 }
 
 void OSDMap::Incremental::dump(Formatter *f) const
@@ -444,6 +493,15 @@ void OSDMap::Incremental::dump(Formatter *f) const
     f->dump_stream("addr") << *p;
   f->close_section();
 
+  f->open_array_section("new_xinfo");
+  for (map<int32_t,osd_xinfo_t>::const_iterator p = new_xinfo.begin(); p != new_xinfo.end(); ++p) {
+    f->open_object_section("xinfo");
+    f->dump_int("osd", p->first);
+    p->second.dump(f);
+    f->close_section();
+  }
+  f->close_section();
+
   if (cluster_snapshot.size())
     f->dump_string("cluster_snapshot", cluster_snapshot);
 
@@ -501,6 +559,7 @@ void OSDMap::set_max_osd(int m)
     osd_weight[o] = CEPH_OSD_OUT;
   }
   osd_info.resize(m);
+  osd_xinfo.resize(m);
   osd_addrs->client_addr.resize(m);
   osd_addrs->cluster_addr.resize(m);
   osd_addrs->hb_addr.resize(m);
@@ -712,6 +771,7 @@ int OSDMap::apply_incremental(Incremental &inc)
     if ((osd_state[i->first] & CEPH_OSD_UP) &&
 	(s & CEPH_OSD_UP)) {
       osd_info[i->first].down_at = epoch;
+      osd_xinfo[i->first].down_stamp = modified;
     }
     if ((osd_state[i->first] & CEPH_OSD_EXISTS) &&
 	(s & CEPH_OSD_EXISTS))
@@ -748,6 +808,10 @@ int OSDMap::apply_incremental(Incremental &inc)
   for (map<int32_t,epoch_t>::iterator p = inc.new_lost.begin(); p != inc.new_lost.end(); p++)
     osd_info[p->first].lost_at = p->second;
 
+  // xinfo
+  for (map<int32_t,osd_xinfo_t>::iterator p = inc.new_xinfo.begin(); p != inc.new_xinfo.end(); ++p)
+    osd_xinfo[p->first] = p->second;
+
   // uuid
   for (map<int32_t,uuid_d>::iterator p = inc.new_uuid.begin(); p != inc.new_uuid.end(); ++p) 
     (*osd_uuid)[p->first] = p->second;
@@ -1032,7 +1096,7 @@ void OSDMap::encode(bufferlist& bl, uint64_t features) const
   ::encode(cbl, bl);
 
   // extended
-  __u16 ev = 8;
+  __u16 ev = 9;
   ::encode(ev, bl);
   ::encode(osd_addrs->hb_addr, bl);
   ::encode(osd_info, bl);
@@ -1041,6 +1105,7 @@ void OSDMap::encode(bufferlist& bl, uint64_t features) const
   ::encode(cluster_snapshot_epoch, bl);
   ::encode(cluster_snapshot, bl);
   ::encode(*osd_uuid, bl);
+  ::encode(osd_xinfo, bl);
 }
 
 void OSDMap::decode(bufferlist& bl)
@@ -1145,6 +1210,10 @@ void OSDMap::decode(bufferlist::iterator& p)
   } else {
     osd_uuid->resize(max_osd);
   }
+  if (ev >= 9)
+    ::decode(osd_xinfo, p);
+  else
+    osd_xinfo.resize(max_osd);
 
   // index pool names
   name_pool.clear();
@@ -1209,6 +1278,17 @@ void OSDMap::dump(Formatter *f) const
     }
   f->close_section();
 
+  f->open_array_section("osd_xinfo");
+  for (int i=0; i<get_max_osd(); i++) {
+    if (exists(i)) {
+      f->open_object_section("xinfo");
+      f->dump_int("osd", i);
+      osd_xinfo[i].dump(f);
+      f->close_section();
+    }
+  }
+  f->close_section();
+
   f->open_array_section("pg_temp");
   for (map<pg_t,vector<int> >::const_iterator p = pg_temp->begin();
        p != pg_temp->end();
@@ -1264,6 +1344,10 @@ string OSDMap::get_flag_string(unsigned f)
     s += ",no-out";
   if (f & CEPH_OSDMAP_NOIN)
     s += ",no-in";
+  if (f & CEPH_OSDMAP_NOBACKFILL)
+    s += ",no-backfill";
+  if (f & CEPH_OSDMAP_NORECOVER)
+    s += ",no-recover";
   if (s.length())
     s = s.erase(0, 1);
   return s;
@@ -1278,7 +1362,7 @@ struct qi {
   int item;
   int depth;
   float weight;
-  qi() {}
+  qi() : item(0), depth(0), weight(0) {}
   qi(int i, int d, float w) : item(i), depth(d), weight(w) {}
 };
 
@@ -1344,23 +1428,51 @@ void OSDMap::print(ostream& out) const
   // ignore pg_swap_primary
 }
 
-void OSDMap::print_osd_line(int cur, ostream& out) const
+void OSDMap::print_osd_line(int cur, ostream *out, Formatter *f) const
 {
-  out << "osd." << cur << "\t";
-  if (!exists(cur))
-    out << "DNE\t\t";
-  else {
-    if (is_up(cur))
-      out << "up\t";
-    else
-      out << "down\t";
-    out << (exists(cur) ? get_weightf(cur):0) << "\t";
+  if (f) {
+    f->dump_unsigned("id", cur);
+    f->dump_stream("name") << "osd." << cur;
+    f->dump_unsigned("exists", (int)exists(cur));
+    f->dump_string("type", crush->get_type_name(0));
+    f->dump_int("type_id", 0);
+  }
+  if (out)
+    *out << "osd." << cur << "\t";
+  if (!exists(cur)) {
+    if (out)
+      *out << "DNE\t\t";
+  } else {
+    if (is_up(cur)) {
+      if (out)
+	*out << "up\t";
+      if (f)
+	f->dump_string("status", "up");
+    } else {
+      if (out)
+	*out << "down\t";
+      if (f)
+	f->dump_string("status", "down");
+    }
+    if (out) {
+      std::streamsize p = out->precision();
+      *out << std::setprecision(4)
+	   << (exists(cur) ? get_weightf(cur) : 0)
+	   << std::setprecision(p)
+	   << "\t";
+    }
+    if (f) {
+      f->dump_float("reweight", get_weightf(cur));
+    }
   }
 }
 
-void OSDMap::print_tree(ostream& out) const
+void OSDMap::print_tree(ostream *out, Formatter *f) const
 {
-  out << "# id\tweight\ttype name\tup/down\treweight\n";
+  if (out)
+    *out << "# id\tweight\ttype name\tup/down\treweight\n";
+  if (f)
+    f->open_array_section("nodes");
   set<int> touched;
   set<int> roots;
   crush->find_roots(roots);
@@ -1373,27 +1485,65 @@ void OSDMap::print_tree(ostream& out) const
       float weight = q.front().weight;
       q.pop_front();
 
-      out << cur << "\t" << weight << "\t";
-      for (int k=0; k<depth; k++)
-	out << "\t";
+      if (out) {
+	*out << cur << "\t";
+	int oldprecision = out->precision();
+	*out << std::setprecision(4) << weight << std::setprecision(oldprecision) << "\t";
 
+	for (int k=0; k<depth; k++)
+	  *out << "\t";
+      }
+      if (f) {
+	f->open_object_section("item");
+      }
       if (cur >= 0) {
-	print_osd_line(cur, out);
-	out << "\n";
+	print_osd_line(cur, out, f);
+	if (out)
+	  *out << "\n";
+	if (f) {
+	  f->close_section();
+	}
 	touched.insert(cur);
+      }
+      if (f) {
+	f->dump_float("crush_weight", weight);
+	f->dump_unsigned("depth", depth);
+      }
+      if (cur >= 0) {
 	continue;
       }
 
-      int type = crush->get_bucket_type(cur);
-      out << crush->get_type_name(type) << " " << crush->get_item_name(cur) << "\n";
-
       // queue bucket contents...
+      int type = crush->get_bucket_type(cur);
       int s = crush->get_bucket_size(cur);
-      for (int k=s-1; k>=0; k--)
-	q.push_front(qi(crush->get_bucket_item(cur, k), depth+1,
-			(float)crush->get_bucket_item_weight(cur, k) / (float)0x10000));
+      if (f) {
+	f->dump_int("id", cur);
+	f->dump_string("name", crush->get_item_name(cur));
+	f->dump_string("type", crush->get_type_name(type));
+	f->dump_int("type_id", type);
+	f->open_array_section("children");
+      }
+      for (int k=s-1; k>=0; k--) {
+	int item = crush->get_bucket_item(cur, k);
+	q.push_front(qi(item, depth+1, (float)crush->get_bucket_item_weight(cur, k) / (float)0x10000));
+	if (f)
+	  f->dump_int("child", item);
+      }
+      if (f)
+	f->close_section();
+
+      if (out)
+	*out << crush->get_type_name(type) << " " << crush->get_item_name(cur) << "\n";
+      if (f) {
+	f->close_section();
+      }
+
     }
   }
+  if (f) {
+    f->close_section();
+    f->open_array_section("stray");
+  }
 
   set<int> stray;
   for (int i=0; i<max_osd; i++)
@@ -1401,14 +1551,22 @@ void OSDMap::print_tree(ostream& out) const
       stray.insert(i);
 
   if (!stray.empty()) {
-    out << "\n";
+    if (out)
+      *out << "\n";
+    if (f)
+      f->open_object_section("osd");
     for (set<int>::iterator p = stray.begin(); p != stray.end(); ++p) {
-      out << *p << "\t0\t";
-      print_osd_line(*p, out);
-      out << "\n";
+      if (out)
+	*out << *p << "\t0\t";
+      print_osd_line(*p, out, f);
+      if (out)
+	*out << "\n";
     }
+    if (f)
+      f->close_section();
   }
-
+  if (f)
+    f->close_section();
 }
 
 void OSDMap::print_summary(ostream& out) const
@@ -1484,7 +1642,7 @@ void OSDMap::build_simple_crush_map(CephContext *cct, CrushWrapper& crush,
   crush.set_type_name(3, "row");
   crush.set_type_name(4, "room");
   crush.set_type_name(5, "datacenter");
-  crush.set_type_name(6, "pool");
+  crush.set_type_name(6, "root");
 
   // root
   int rootid = crush.add_bucket(0, CRUSH_BUCKET_STRAW, CRUSH_HASH_DEFAULT, 6 /* pool */, 0, NULL, NULL);
@@ -1494,7 +1652,7 @@ void OSDMap::build_simple_crush_map(CephContext *cct, CrushWrapper& crush,
     map<string,string> loc;
     loc["host"] = "localhost";
     loc["rack"] = "localrack";
-    loc["pool"] = "default";
+    loc["root"] = "default";
     ldout(cct, 10) << " adding osd." << o << " at " << loc << dendl;
     char name[8];
     sprintf(name, "osd.%d", o);
@@ -1606,7 +1764,7 @@ void OSDMap::build_simple_crush_map_from_conf(CephContext *cct, CrushWrapper& cr
   crush.set_type_name(3, "row");
   crush.set_type_name(4, "room");
   crush.set_type_name(5, "datacenter");
-  crush.set_type_name(6, "pool");
+  crush.set_type_name(6, "root");
 
   set<string> hosts, racks;
 
@@ -1636,7 +1794,7 @@ void OSDMap::build_simple_crush_map_from_conf(CephContext *cct, CrushWrapper& cr
     conf->get_val_from_conf_file(sections, "row", row, false);
     conf->get_val_from_conf_file(sections, "room", room, false);
     conf->get_val_from_conf_file(sections, "datacenter", dc, false);
-    conf->get_val_from_conf_file(sections, "pool", pool, false);
+    conf->get_val_from_conf_file(sections, "root", pool, false);
 
     if (host.length() == 0)
       host = "unknownhost";
@@ -1655,7 +1813,7 @@ void OSDMap::build_simple_crush_map_from_conf(CephContext *cct, CrushWrapper& cr
       loc["room"] = room;
     if (dc.size())
       loc["datacenter"] = dc;
-    loc["pool"] = "default";
+    loc["root"] = "default";
 
     ldout(cct, 5) << " adding osd." << o << " at " << loc << dendl;
     crush.insert_item(cct, o, 1.0, *i, loc);
