@@ -30,11 +30,13 @@ class RGWOp {
 protected:
   struct req_state *s;
   RGWHandler *dialect_handler;
+  RGWRados *store;
 public:
-  RGWOp() {}
+  RGWOp() : s(NULL), dialect_handler(NULL), store(NULL) {}
   virtual ~RGWOp() {}
 
-  virtual void init(struct req_state *s, RGWHandler *dialect_handler) {
+  virtual void init(RGWRados *store, struct req_state *s, RGWHandler *dialect_handler) {
+    this->store = store;
     this->s = s;
     this->dialect_handler = dialect_handler;
   }
@@ -42,6 +44,8 @@ public:
   virtual bool prefetch_data() { return false; }
   virtual int verify_permission() = 0;
   virtual void execute() = 0;
+  virtual void send_response() {}
+  virtual void complete() { send_response(); }
   virtual const char *name() = 0;
 };
 
@@ -70,12 +74,7 @@ protected:
 
   int init_common();
 public:
-  RGWGetObj() {}
-
-  virtual bool prefetch_data() { return true; }
-
-  virtual void init(struct req_state *s, RGWHandler *h) {
-    RGWOp::init(s, h);
+  RGWGetObj() {
     range_str = NULL;
     if_mod = NULL;
     if_unmod = NULL;
@@ -91,17 +90,22 @@ public:
     unmod_time = 0;
     mod_ptr = NULL;
     unmod_ptr = NULL;
-    attrs.clear();
+    get_data = false;
     partial_content = false;
     ret = 0;
+ }
+
+  virtual bool prefetch_data() { return true; }
 
-    /* get_data should not be initialized here! */
-  }
   void set_get_data(bool get_data) {
     this->get_data = get_data;
   }
   int verify_permission();
   void execute();
+  int read_user_manifest_part(rgw_bucket& bucket, RGWObjEnt& ent, RGWAccessControlPolicy *bucket_policy, off_t start_ofs, off_t end_ofs);
+  int iterate_user_manifest_parts(rgw_bucket& bucket, string& obj_prefix, RGWAccessControlPolicy *bucket_policy,
+                                  uint64_t *ptotal_len, bool read_data);
+  int handle_user_manifest(const char *prefix);
 
   virtual int get_params() = 0;
   virtual int send_response(bufferlist& bl) = 0;
@@ -115,16 +119,12 @@ protected:
   RGWUserBuckets buckets;
 
 public:
-  virtual void init(struct req_state *s, RGWHandler *h) {
-    RGWOp::init(s, h);
-    buckets.clear();
-  }
-  RGWListBuckets() {}
+  RGWListBuckets() : ret(0) {}
 
   int verify_permission();
   void execute();
 
-  virtual int get_params()= 0;
+  virtual int get_params() = 0;
   virtual void send_response() = 0;
 
   virtual const char *name() { return "list_buckets"; }
@@ -139,15 +139,13 @@ protected:
   uint64_t buckets_size_rounded;
 
 public:
-  virtual void init(struct req_state *s, RGWHandler *h) {
-    RGWOp::init(s, h);
+  RGWStatAccount() {
     ret = 0;
     buckets_count = 0;
     buckets_objcount = 0;
     buckets_size = 0;
     buckets_size_rounded = 0;
   }
-  RGWStatAccount() {}
 
   int verify_permission();
   void execute();
@@ -173,18 +171,10 @@ protected:
   int parse_max_keys();
 
 public:
-  RGWListBucket() {}
-
-  virtual void init(struct req_state *s, RGWHandler *h) {
-    RGWOp::init(s, h);
-    prefix.clear();
-    marker.clear();
-    max_keys.clear();
-    delimiter.clear();
+  RGWListBucket() {
     max = 0;
     ret = 0;
-    objs.clear();
-    common_prefixes.clear();
+    default_max = 0;
     is_truncated = false;
   }
   int verify_permission();
@@ -195,18 +185,23 @@ public:
   virtual const char *name() { return "list_bucket"; }
 };
 
+class RGWGetBucketLogging : public RGWOp {
+public:
+  RGWGetBucketLogging() {}
+  int verify_permission();
+  void execute() {}
+
+  virtual void send_response() = 0;
+  virtual const char *name() { return "get_bucket_logging"; }
+};
+
 class RGWStatBucket : public RGWOp {
 protected:
   int ret;
   RGWBucketEnt bucket;
 
 public:
-  virtual void init(struct req_state *s, RGWHandler *h) {
-    RGWOp::init(s, h);
-    ret = 0;
-    bucket.clear();
-  }
-  RGWStatBucket() {}
+  RGWStatBucket() : ret(0) {}
   ~RGWStatBucket() {}
 
   int verify_permission();
@@ -222,14 +217,13 @@ protected:
   RGWAccessControlPolicy policy;
 
 public:
-  RGWCreateBucket() {}
+  RGWCreateBucket() : ret(0) {}
 
   int verify_permission();
   void execute();
-  virtual void init(struct req_state *s, RGWHandler *h) {
-    RGWOp::init(s, h);
+  virtual void init(RGWRados *store, struct req_state *s, RGWHandler *h) {
+    RGWOp::init(store, s, h);
     policy.set_ctx(s->cct);
-    ret = 0;
   }
   virtual int get_params() { return 0; }
   virtual void send_response() = 0;
@@ -241,12 +235,8 @@ protected:
   int ret;
 
 public:
-  RGWDeleteBucket() {}
+  RGWDeleteBucket() : ret(0) {}
 
-  virtual void init(struct req_state *s, RGWHandler *h) {
-    RGWOp::init(s, h);
-    ret = 0;
-  }
   int verify_permission();
   void execute();
 
@@ -257,10 +247,12 @@ public:
 class RGWPutObjProcessor
 {
 protected:
+  RGWRados *store;
   struct req_state *s;
 public:
   virtual ~RGWPutObjProcessor() {}
-  virtual int prepare(struct req_state *_s) {
+  virtual int prepare(RGWRados *_store, struct req_state *_s) {
+    store = _store;
     s = _s;
     return 0;
   };
@@ -281,18 +273,20 @@ protected:
   string etag;
   bool chunked_upload;
   RGWAccessControlPolicy policy;
+  const char *obj_manifest;
 
 public:
-  RGWPutObj() {}
-
-  virtual void init(struct req_state *s, RGWHandler *h) {
-    RGWOp::init(s, h);
+  RGWPutObj() {
     ret = 0;
     ofs = 0;
     supplied_md5_b64 = NULL;
     supplied_etag = NULL;
-    etag = "";
     chunked_upload = false;
+    obj_manifest = NULL;
+  }
+
+  virtual void init(RGWRados *store, struct req_state *s, RGWHandler *h) {
+    RGWOp::init(store, s, h);
     policy.set_ctx(s->cct);
   }
 
@@ -316,12 +310,13 @@ protected:
   RGWAccessControlPolicy policy;
 
 public:
-  RGWPutMetadata() {}
-
-  virtual void init(struct req_state *s, RGWHandler *h) {
-    RGWOp::init(s, h);
+  RGWPutMetadata() {
     has_policy = false;
     ret = 0;
+  }
+
+  virtual void init(RGWRados *store, struct req_state *s, RGWHandler *h) {
+    RGWOp::init(store, s, h);
     policy.set_ctx(s->cct);
   }
   int verify_permission();
@@ -337,12 +332,8 @@ protected:
   int ret;
 
 public:
-  RGWDeleteObj() {}
+  RGWDeleteObj() : ret(0) {}
 
-  virtual void init(struct req_state *s, RGWHandler *h) {
-    RGWOp::init(s, h);
-    ret = 0;
-  }
   int verify_permission();
   void execute();
 
@@ -373,6 +364,7 @@ protected:
   rgw_bucket dest_bucket;
   string dest_object;
   time_t mtime;
+  bool replace_attrs;
 
   int init_common();
 
@@ -380,10 +372,7 @@ protected:
   bool parse_copy_location(const char *src, string& bucket_name, string& object);
 
 public:
-  RGWCopyObj() {}
-
-  virtual void init(struct req_state *s, RGWHandler *h) {
-    RGWOp::init(s, h);
+  RGWCopyObj() {
     if_mod = NULL;
     if_unmod = NULL;
     if_match = NULL;
@@ -391,13 +380,17 @@ public:
     ofs = 0;
     len = 0;
     end = -1;
+    mod_time = 0;
+    unmod_time = 0;
     mod_ptr = NULL;
     unmod_ptr = NULL;
     ret = 0;
-    attrs.clear();
-    src_bucket.clear();
-    src_object.clear();
     mtime = 0;
+    replace_attrs = false;
+  }
+
+  virtual void init(RGWRados *store, struct req_state *s, RGWHandler *h) {
+    RGWOp::init(store, s, h);
     dest_policy.set_ctx(s->cct);
   }
   int verify_permission();
@@ -415,13 +408,8 @@ protected:
   string acls;
 
 public:
-  RGWGetACLs() {}
+  RGWGetACLs() : ret(0) {}
 
-  virtual void init(struct req_state *s, RGWHandler *h) {
-    RGWOp::init(s, h);
-    ret = 0;
-    acls.clear();
-  }
   int verify_permission();
   void execute();
 
@@ -436,14 +424,12 @@ protected:
   char *data;
 
 public:
-  RGWPutACLs() {}
-
-  virtual void init(struct req_state *s, RGWHandler *h) {
-    RGWOp::init(s, h);
+  RGWPutACLs() {
     ret = 0;
     len = 0;
     data = NULL;
   }
+
   int verify_permission();
   void execute();
 
@@ -460,12 +446,12 @@ protected:
   RGWAccessControlPolicy policy;
 
 public:
-  RGWInitMultipart() {}
-
-  virtual void init(struct req_state *s, RGWHandler *h) {
-    RGWOp::init(s, h);
+  RGWInitMultipart() {
     ret = 0;
-    upload_id = "";
+  }
+
+  virtual void init(RGWRados *store, struct req_state *s, RGWHandler *h) {
+    RGWOp::init(store, s, h);
     policy.set_ctx(s->cct);
   }
   int verify_permission();
@@ -485,16 +471,12 @@ protected:
   int len;
 
 public:
-  RGWCompleteMultipart() {}
-
-  virtual void init(struct req_state *s, RGWHandler *h) {
-    RGWOp::init(s, h);
+  RGWCompleteMultipart() {
     ret = 0;
-    upload_id = "";
-    etag="";
     data = NULL;
     len = 0;
   }
+
   int verify_permission();
   void execute();
 
@@ -508,12 +490,8 @@ protected:
   int ret;
 
 public:
-  RGWAbortMultipart() {}
+  RGWAbortMultipart() : ret(0) {}
 
-  virtual void init(struct req_state *s, RGWHandler *h) {
-    RGWOp::init(s, h);
-    ret = 0;
-  }
   int verify_permission();
   void execute();
 
@@ -531,15 +509,14 @@ protected:
   RGWAccessControlPolicy policy;
 
 public:
-  RGWListMultipart() {}
-
-  virtual void init(struct req_state *s, RGWHandler *h) {
-    RGWOp::init(s, h);
+  RGWListMultipart() {
     ret = 0;
-    upload_id = "";
-    parts.clear();
     max_parts = 1000;
     marker = 0;
+  }
+
+  virtual void init(RGWRados *store, struct req_state *s, RGWHandler *h) {
+    RGWOp::init(store, s, h);
     policy = RGWAccessControlPolicy(s->cct);
   }
   int verify_permission();
@@ -640,21 +617,16 @@ protected:
   int default_max;
 
 public:
-  RGWListBucketMultiparts() {}
-
-  virtual void init(struct req_state *s, RGWHandler *h) {
-    RGWOp::init(s, h);
-    prefix.clear();
-    marker.clear();
-    next_marker.clear();
-    max_uploads = default_max;
-    delimiter.clear();
-    max_uploads = default_max;
+  RGWListBucketMultiparts() {
     ret = 0;
-    uploads.clear();
     is_truncated = false;
-    common_prefixes.clear();
   }
+
+  virtual void init(RGWRados *store, struct req_state *s, RGWHandler *h) {
+    RGWOp::init(store, s, h);
+    max_uploads = default_max;
+  }
+
   int verify_permission();
   void execute();
 
@@ -663,18 +635,59 @@ public:
   virtual const char *name() { return "list_bucket_multiparts"; }
 };
 
+class RGWDeleteMultiObj : public RGWOp {
+protected:
+  int ret;
+  int max_to_delete;
+  size_t len;
+  char *data;
+  string bucket_name;
+  rgw_bucket bucket;
+  bool quiet;
+  bool status_dumped;
+
+
+public:
+  RGWDeleteMultiObj() {
+    ret = 0;
+    max_to_delete = 1000;
+    len = 0;
+    data = NULL;
+    quiet = false;
+    status_dumped = false;
+  }
+  int verify_permission();
+  void execute();
+
+  virtual int get_params() = 0;
+  virtual void send_status() = 0;
+  virtual void begin_response() = 0;
+  virtual void send_partial_response(pair<string,int>& result) = 0;
+  virtual void end_response() = 0;
+  virtual const char *name() { return "multi_object_delete"; }
+};
+
+
 class RGWHandler {
 protected:
+  RGWRados *store;
   struct req_state *s;
 
   int do_read_permissions(RGWOp *op, bool only_bucket);
-public:
-  RGWHandler() {}
-  virtual ~RGWHandler() {}
-  virtual int init(struct req_state *_s, FCGX_Request *fcgx);
 
-  virtual RGWOp *get_op() = 0;
-  virtual void put_op(RGWOp *op) = 0;
+  virtual RGWOp *op_get() { return NULL; }
+  virtual RGWOp *op_put() { return NULL; }
+  virtual RGWOp *op_delete() { return NULL; }
+  virtual RGWOp *op_head() { return NULL; }
+  virtual RGWOp *op_post() { return NULL; }
+  virtual RGWOp *op_copy() { return NULL; }
+public:
+  RGWHandler() : store(NULL), s(NULL) {}
+  virtual ~RGWHandler();
+  virtual int init(RGWRados *store, struct req_state *_s, RGWClientIO *cio);
+
+  virtual RGWOp *get_op(RGWRados *store);
+  virtual void put_op(RGWOp *op);
   virtual int read_permissions(RGWOp *op) = 0;
   virtual int authorize() = 0;
 };
